@@ -1,14 +1,11 @@
-#ifndef lint
-static char rcsId[]="$Header: /cvs/gnome/gnome-libs/gtk-xmhtml/readJPEG.c,v 1.4 1998/02/12 03:09:46 unammx Exp $";
-#endif
 /*****
 * readJPEG.c : XmHTML jpeg image loading from a memory buffer
 *
-* This file Version	$Revision: 1.4 $
+* This file Version	$Revision: 1.5 $
 *
 * Creation date:		Wed Feb 19 03:13:58 GMT+0100 1997
-* Last modification: 	$Date: 1998/02/12 03:09:46 $
-* By:					$Author: unammx $
+* Last modification: 	$Date: 1999/07/29 01:26:29 $
+* By:					$Author: sopwith $
 * Current State:		$State: Exp $
 *
 * Author:				newt
@@ -37,6 +34,11 @@ static char rcsId[]="$Header: /cvs/gnome/gnome-libs/gtk-xmhtml/readJPEG.c,v 1.4
 /*****
 * ChangeLog 
 * $Log: readJPEG.c,v $
+* Revision 1.5  1999/07/29 01:26:29  sopwith
+*
+*
+* Fix all warnings.
+*
 * Revision 1.4  1998/02/12 03:09:46  unammx
 * Merge to Koen's XmHTML 1.1.2 + following fixes:
 *
